@@ -18,7 +18,6 @@ final class WP_Comment {
 	 * Comment ID.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var int
 	 */
 	public $comment_ID;
@@ -27,7 +26,6 @@ final class WP_Comment {
 	 * ID of the post the comment is associated with.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var int
 	 */
 	public $comment_post_ID = 0;
@@ -36,7 +34,6 @@ final class WP_Comment {
 	 * Comment author name.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $comment_author = '';
@@ -45,7 +42,6 @@ final class WP_Comment {
 	 * Comment author email address.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $comment_author_email = '';
@@ -54,7 +50,6 @@ final class WP_Comment {
 	 * Comment author URL.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $comment_author_url = '';
@@ -63,7 +58,6 @@ final class WP_Comment {
 	 * Comment author IP address (IPv4 format).
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $comment_author_IP = '';
@@ -72,7 +66,6 @@ final class WP_Comment {
 	 * Comment date in YYYY-MM-DD HH:MM:SS format.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $comment_date = '0000-00-00 00:00:00';
@@ -81,7 +74,6 @@ final class WP_Comment {
 	 * Comment GMT date in YYYY-MM-DD HH::MM:SS format.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $comment_date_gmt = '0000-00-00 00:00:00';
@@ -90,7 +82,6 @@ final class WP_Comment {
 	 * Comment content.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $comment_content;
@@ -99,7 +90,6 @@ final class WP_Comment {
 	 * Comment karma count.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var int
 	 */
 	public $comment_karma = 0;
@@ -108,7 +98,6 @@ final class WP_Comment {
 	 * Comment approval status.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $comment_approved = '1';
@@ -117,7 +106,6 @@ final class WP_Comment {
 	 * Comment author HTTP user agent.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $comment_agent = '';
@@ -126,7 +114,6 @@ final class WP_Comment {
 	 * Comment type.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $comment_type = '';
@@ -135,7 +122,6 @@ final class WP_Comment {
 	 * Parent comment ID.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var int
 	 */
 	public $comment_parent = 0;
@@ -144,7 +130,6 @@ final class WP_Comment {
 	 * Comment author ID.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var int
 	 */
 	public $user_id = 0;
@@ -153,7 +138,6 @@ final class WP_Comment {
 	 * Comment children.
 	 *
 	 * @since 4.4.0
-	 * @access protected
 	 * @var array
 	 */
 	protected $children;
@@ -162,7 +146,6 @@ final class WP_Comment {
 	 * Whether children have been populated for this comment object.
 	 *
 	 * @since 4.4.0
-	 * @access protected
 	 * @var bool
 	 */
 	protected $populated_children = false;
@@ -171,7 +154,6 @@ final class WP_Comment {
 	 * Post fields.
 	 *
 	 * @since 4.4.0
-	 * @access protected
 	 * @var array
 	 */
 	protected $post_fields = array( 'post_author', 'post_date', 'post_date_gmt', 'post_content', 'post_title', 'post_excerpt', 'post_status', 'comment_status', 'ping_status', 'post_name', 'to_ping', 'pinged', 'post_modified', 'post_modified_gmt', 'post_content_filtered', 'post_parent', 'guid', 'menu_order', 'post_type', 'post_mime_type', 'comment_count' );
@@ -180,7 +162,6 @@ final class WP_Comment {
 	 * Retrieves a WP_Comment instance.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @static
 	 *
 	 * @global wpdb $wpdb WordPress database abstraction object.
@@ -191,12 +172,11 @@ final class WP_Comment {
 	public static function get_instance( $id ) {
 		global $wpdb;
 
-		if ( ! is_numeric( $id ) || $id != floor( $id ) || ! $id ) {
+		$comment_id = (int) $id;
+		if ( ! $comment_id ) {
 			return false;
 		}
 
-		$comment_id = (int) $id;
-
 		$_comment = wp_cache_get( $comment_id, 'comment' );
 
 		if ( ! $_comment ) {
@@ -218,7 +198,6 @@ final class WP_Comment {
 	 * Populates properties with object vars.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 *
 	 * @param WP_Comment $comment Comment object.
 	 */
@@ -232,7 +211,6 @@ final class WP_Comment {
 	 * Convert object to array.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 *
 	 * @return array Object as array.
 	 */
@@ -244,7 +222,6 @@ final class WP_Comment {
 	 * Get the children of a comment.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 *
 	 * @param array $args {
 	 *     Array of arguments used to pass to get_comments() and determine format.
@@ -319,7 +296,6 @@ final class WP_Comment {
 	 * Used by `WP_Comment_Query` when bulk-filling descendants.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 *
 	 * @param WP_Comment $child Child comment.
 	 */
@@ -331,7 +307,6 @@ final class WP_Comment {
 	 * Get a child comment by ID.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 *
 	 * @param int $child_id ID of the child.
 	 * @return WP_Comment|bool Returns the comment object if found, otherwise false.
@@ -364,7 +339,6 @@ final class WP_Comment {
 	 * If `$name` matches a post field, the comment post will be loaded and the post's value checked.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 *
 	 * @param string $name Property name.
 	 * @return bool
@@ -382,7 +356,6 @@ final class WP_Comment {
 	 * If `$name` matches a post field, the comment post will be loaded and the post's value returned.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 *
 	 * @param string $name
 	 * @return mixed
