@@ -85,10 +85,11 @@
 		 * and creates a new instance for every match.
 		 *
 		 * @param {String} content The string to scan.
+		 * @param {tinymce.Editor} editor The editor.
 		 *
 		 * @return {String} The string with markers.
 		 */
-		setMarkers: function( content ) {
+		setMarkers: function( content, editor ) {
 			var pieces = [ { content: content } ],
 				self = this,
 				instance, current;
@@ -115,6 +116,7 @@
 							pieces.push( { content: remaining.substring( 0, result.index ) } );
 						}
 
+						result.options.editor = editor;
 						instance = self.createInstance( type, result.content, result.options );
 						text = instance.loader ? '.' : instance.text;
 
@@ -278,7 +280,7 @@
 
 	wp.mce.View.extend = Backbone.View.extend;
 
-	_.extend( wp.mce.View.prototype, {
+	_.extend( wp.mce.View.prototype, /** @lends wp.mce.View.prototype */{
 
 		/**
 		 * The content.
@@ -850,7 +852,7 @@
 		action: 'parse-media-shortcode',
 
 		initialize: function() {
-			var self = this;
+			var self = this, maxwidth = null;
 
 			if ( this.url ) {
 				this.loader = false;
@@ -859,10 +861,16 @@
 				} );
 			}
 
+			// Obtain the target width for the embed.
+			if ( self.editor ) {
+				maxwidth = self.editor.getBody().clientWidth;
+			}
+
 			wp.ajax.post( this.action, {
 				post_ID: media.view.settings.post.id,
 				type: this.shortcode.tag,
-				shortcode: this.shortcode.string()
+				shortcode: this.shortcode.string(),
+				maxwidth: maxwidth
 			} )
 			.done( function( response ) {
 				self.render( response );
